@@ -22,24 +22,20 @@ can be computed here, so the code generation can be quick and doesn't have to
 check it many times.
 
 """
-from .FinalizeMarkups import FinalizeMarkups
+from nuitka.tree import Operations
+
 from .FinalizeClosureTaking import FinalizeClosureTaking
-from .FinalizeVariableVisibility import FinalizeVariableVisibility
+from .FinalizeMarkups import FinalizeMarkups
+
 
 # Bug of pylint, it's there but it reports it wrongly, pylint: disable=E0611
-from nuitka.tree import Operations
 
 def prepareCodeGeneration(tree):
     visitor = FinalizeMarkups()
-    Operations.visitTree( tree, visitor )
+    Operations.visitTree(tree, visitor)
     for function in tree.getUsedFunctions():
-        Operations.visitTree( function, visitor )
+        Operations.visitTree(function, visitor)
 
     visitor = FinalizeClosureTaking()
     for function in tree.getUsedFunctions():
-        Operations.visitFunction( function, visitor )
-
-    visitor = FinalizeVariableVisibility()
-    Operations.visitFunction( tree, visitor )
-    for function in tree.getUsedFunctions():
-        Operations.visitFunction( function, visitor )
+        Operations.visitFunction(function, visitor)
