@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,23 +20,23 @@ Namespace packages of Python3.3
 
 """
 
-from nuitka.nodes.ModuleNodes import PythonPackage
-
-from nuitka.SourceCodeReferences import SourceCodeReference
-from nuitka.nodes.FutureSpecs import FutureSpec
-
-from nuitka.tree.Helpers import makeStatementsSequenceFromStatement
 from nuitka.nodes.AssignNodes import StatementAssignmentVariable
-from nuitka.nodes.VariableRefNodes import ExpressionTargetVariableRef
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
-from nuitka.tree.VariableClosure import completeVariableClosures
 from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
+from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.FutureSpecs import FutureSpec
 from nuitka.nodes.ImportNodes import (
-    ExpressionImportName,
-    ExpressionImportModule
+    ExpressionImportModule,
+    ExpressionImportName
 )
+from nuitka.nodes.ModuleNodes import PythonPackage
+from nuitka.nodes.VariableRefNodes import ExpressionTargetVariableRef
+from nuitka.SourceCodeReferences import SourceCodeReference
 
-def createNamespacePackage( package_name, module_relpath ):
+from .Helpers import makeStatementsSequenceFromStatement
+from .VariableClosure import completeVariableClosures
+
+
+def createNamespacePackage(package_name, module_relpath):
     parts = package_name.split(".")
 
     source_ref = SourceCodeReference.fromFilenameAndLine(
